from typing import Dict, List, Tuple, Optional, BinaryIO
from .__http_client import _HTTPClient
import mimetypes
import os
import requests


class _Files:
    """Files resource for managing file uploads and operations."""

    def __init__(self, http_client: _HTTPClient):
        self._http_client = http_client

    def upload(self, files: List[str], knowledge_id: Optional[str] = None) -> List[Dict]:
        """Upload files to AI Library.
        files is a list where each element contains a path to the file.
        """
        key = self._http_client.headers["X-Library-Key"]
        domain = self._http_client.base_url
        url = domain + "/v1/files"
        headers = {
            'X-Library-Key': key
        }
        payload = {}
        if knowledge_id:
            payload['knowledgeId'] = knowledge_id
        for file in files:
            file_name = os.path.basename(file)
            mime_type = mimetypes.guess_type(file)[0]
            files = [
                ('files', (file_name, open(file, 'rb'), mime_type))
            ]

        res = requests.request(
            "POST", url, headers=headers, data=payload, files=files)
        return res.text

    def list_files(self, page: Optional[int] = None, limit: Optional[int] = None) -> Dict:
        """List all files."""
        params_dict = {}
        optional_params = {"page": page, "limit": limit}
        for param in optional_params:
            param_value = optional_params[param]
            if param_value is not None:
                params_dict[param] = param_value

        return self._http_client._request("GET", "/v1/files", params=params_dict)

    def get(self, file_id: str) -> Dict:
        """Retrieve a file by ID."""
        return self._http_client._request("GET", f"/v1/files/{file_id}")

    def delete(self, file_id: str) -> Dict:
        """Delete a file."""
        return self._http_client._request("DELETE", f"/v1/files/{file_id}")



from ._internal._agent import _Agent
from ._internal._knowledge_base import _KnowledgeBase
from ._internal._files import _Files
from ._internal._utilities import _Utilities
from ._internal._notes import _Notes
from ._internal.__http_client import _HTTPClient


class AILibrary:
    """Main client for interacting with the AI Library API."""

    def __init__(self, api_key: str, domain: str = "https://api.ailibrary.ai/"):
        self._http_client = _HTTPClient(api_key, domain)

        # Initialize resources
        self.agent = _Agent(self._http_client)
        self.knowledge_base = _KnowledgeBase(self._http_client)
        self.files = _Files(self._http_client)
        self.utilities = _Utilities(self._http_client)
        self.notes = _Notes(self._http_client)



from typing import Dict, List, Optional, Literal
from .__http_client import _HTTPClient
import json
import requests


class _Agent:
    """Client for interacting with the AI Library Agent API."""

    def __init__(self, http_client: _HTTPClient):
        self._http_client = http_client

    def create(
        self,
        title: str,
        instructions: Optional[str] = "You are a helpful assistant.",
        description: Optional[str] = None,
        coverimage: Optional[str] = None,
        intromessage: Optional[str] = None,
        knowledge_search: Optional[bool] = None,
        knowledge_id: Optional[str] = None
    ) -> Dict:
        """Create a new agent with the specified parameters."""

        if not title:
            raise ValueError("Title cannot be empty")
        payload = {"title": title}
        optional_params = {"instructions": instructions, "description": description,
                           "coverimage": coverimage, "intromessage": intromessage,
                           "knowledge_search": knowledge_search, "knowledge_id": knowledge_id}
        for param in optional_params:
            param_value = optional_params[param]
            if param_value is not None:
                payload[param] = param_value
        return self._http_client._request("POST", "/v1/agent/create", json=payload)

    def get(self, namespace: str) -> Dict:
        """Retrieve information about an agent."""
        return self._http_client._request("GET", f"/v1/agent/{namespace}")

    def list_agents(self) -> Dict:
        """List all agents."""
        return self._http_client._request("GET", "/v1/agent")

    def update(
        self,
        namespace: str,
        title: Optional[str] = None,
        type: Optional[Literal["notebook", "chat", "voice"]] = None,
        instructions: Optional[str] = "You are a helpful assistant.",
        description: Optional[str] = None,
        coverimage: Optional[str] = None,
        intromessage: Optional[str] = None,
        knowledge_search: Optional[bool] = None,
        knowledge_id: Optional[str] = None
    ) -> Dict:
        """Update an existing agent."""

        payload = {"namespace": namespace}
        valid_types = ["notebook", "chat", "voice"]

        optional_params = {"title": title, "type": type, "instructions": instructions,
                           "description": description, "coverimage": coverimage,
                           "intromessage": intromessage, "knowledge_search": knowledge_search,
                           "knowledge_id": knowledge_id}
        for param in optional_params:
            param_value = optional_params[param]
            if param == "type" and param_value and param_value not in valid_types:
                raise ValueError(f"Invalid agent type. If specified, must be one of: {
                                 self._http_client._stringify(valid_types)} .")
            elif param_value is not None:
                payload[param] = param_value
        return self._http_client._request("PUT", f"/v1/agent/{namespace}", json=payload)

    def delete(self, namespace: str) -> Dict:
        """Delete an agent."""
        return self._http_client._request("DELETE", f"/v1/agent/{namespace}")

    def chat(self, namespace: str, messages: List[Dict[str, str]], session_id: Optional[str] = None) -> Dict:
        """Chat with an agent.

        Args:
            namespace: The agent namespace
            messages: List of message dictionaries (at least one).
                Requirements:
                    - At least one message
                    - Required key: 'role' 
                        - Possible values: 'assistant', 'user', 'system'
                    - Required key: 'content'
                        - Possible values: any string
            session_id: Optional session identifier
        """
        if not namespace:
            raise ValueError("Namespace cannot be empty")
        if not messages:
            raise ValueError("Messages list cannot be empty")

        valid_roles = {"assistant", "user", "system"}
        for msg in messages:
            if not isinstance(msg, dict):
                raise ValueError("Each message must be a dictionary")
            if "role" not in msg or "content" not in msg:
                raise ValueError(
                    "Each message must contain 'role' and 'content' keys")
            if msg["role"] not in valid_roles:
                raise ValueError(f"Message role must be one of {valid_roles}")
            if not isinstance(msg["content"], str):
                raise ValueError("Message content must be a string")

        domain = self._http_client.base_url
        url = f"{domain}/v1/agent/{namespace}/chat"
        payload = json.dumps({
            "messages": messages,
            "session_id": session_id if session_id else "test-session",
        })
        headers = {
            'Content-Type': 'application/json',
            'X-Library-Key': self._http_client.headers["X-Library-Key"]
        }
        # response = requests.request("POST", url, headers=headers, data=payload)
        with requests.request("POST", url, headers=headers, data=payload, stream=True) as response:
            response.raise_for_status()
            for chunk in response.iter_content(chunk_size=8192):
                if chunk:
                    decoded_chunk = chunk.decode('utf-8')
                    yield decoded_chunk
        



import requests
from typing import Dict, List, Tuple, Optional, Any, BinaryIO


class _HTTPClient:
    """Handles HTTP requests to the AI Library API."""
    
    def __init__(self, api_key: str, base_url: str):
        if base_url[-1] == "/":
            self.base_url = base_url[:-1]
        else:
            self.base_url = base_url

        self.headers = {
            "X-Library-Key": api_key,
            "Content-Type": "application/json"
        }
    

    def _stringify(list_of_strings: str):
        """ 
        input example: ["A", "list", "of", "words"]
        return value example: "'A', 'list', 'of', 'words'"
        """
        return "'" + "', '".join(list_of_strings) + "'"


    def _request(
        self, 
        method: str, 
        endpoint: str, 
        params: Optional[Dict] = None,
        json: Optional[Dict] = None,
        files: Optional[List[Tuple[str, Tuple[str, BinaryIO, str]]]] = None,
        stream: bool = False,
        # response_no_json: bool = False
    ) -> Any:
        """Make an HTTP request to the API."""

        url = f"{self.base_url}{endpoint}"
        response = requests.request(
            method=method,
            url=url,
            headers=self.headers,
            params=params,
            json=json,
            files=files,
            stream=stream
        )

        # if response_no_json:
        #     return response
        response.raise_for_status()
        return response.json()



from typing import Dict, List, Optional
from .__http_client import _HTTPClient


class _Notes:
    """Notes resource for managing notes on resources."""

    def __init__(self, http_client: _HTTPClient):
        self.VALID_ROLES = ["assistant", "user", "system"]
        self.VALID_RESOURCES = ["agent", "knowledgebase", "file"]
        self._http_client = http_client


    def _check_role(self, role: str):
        if role not in self.VALID_ROLES:
            raise ValueError(f"Invalid role. Valid roles: {self._http_client._stringify(self.VALID_ROLES)} .")


    def _check_resource(self, resource: str):
        if resource not in self.VALID_RESOURCES:
            raise ValueError(f"Invalid resource. Valid resources: {self._http_client._stringify(self.VALID_RESOURCES)} .")


    def add(
        self,
        content: str,
        role: str,
        resource: str,
        resource_id: str,
        meta: Optional[Dict] = None
    ) -> Dict:
        """Add a note to a resource.
            Args:
                content: The content of the note
                role: Possible values: see self.VALID_ROLES
                resource: Possible values: see self.VALID_RESOURCES
                resource_id:
                    if resouce == 'agent':
                        resource_id is namespace
                    if resouce == 'knowledgebase'
                        resource_id is knowledgeId
                    if resouce == 'file'
                        resource_id is id

                meta: Optional arg
        """

        self._check_role(role)
        self._check_resource(resource)
        
        payload = {
            "content": content,
            "role": role,
            "resource": resource,
            "resource_id": resource_id
        }
        if meta:
            payload["meta"] = meta
        return self._http_client._request("POST", "/v1/notes", json=payload)


    def get_for_resource(self, resource: str, resource_id: str) -> List[Dict]:
        """Get notes for a resource."""
        self._check_resource(resource)
        return self._http_client._request("GET", f"/v1/notes/{resource}/{resource_id}")


    def update(
        self,
        note_id: str,
        content: str,
        role: str,
        meta: Optional[Dict] = None
    ) -> Dict:
        """Update a note."""
        self._check_role(role)
        payload = {
            "content": content,
            "role": role
        }
        if meta:
            payload["meta"] = meta
        return self._http_client._request("PUT", f"/v1/notes/{note_id}", json=payload)


    def get(self, note_id: str) -> Dict:
        """Get a note by ID."""
        return self._http_client._request("GET", f"/v1/notes/{note_id}")


    def delete_notes(
        self,
        resource: str,
        resource_id: str,
        values: List[str] = None,
        delete_all: bool = None
    ) -> Dict:
        """Delete notes for a resource.
        <resource> and <resource_id> are required, as well as 
        one of the following: <values> or <delete_all>.
        If <values> is not provided, <delete_all> must be true.
        """
        self._check_resource(resource)
        if (not values and not delete_all):
            raise ValueError("One of the following is required: values or delete_all=True.")
        
        payload = {"resource": resource, "resource_id": resource_id}
        if delete_all:
            payload["delete_all"] = delete_all  # doesn't matter what values are, delete all notes
        else:
            payload["values"] = values # only delete notes with these values
        return self._http_client._request("DELETE", f"/v1/notes/{resource}/{resource_id}", json=payload)



from typing import Dict, List
from .__http_client import _HTTPClient


class _Utilities:
    """Utility functions to support AI agents."""

    def __init__(self, http_client: _HTTPClient):
        self._http_client = http_client


    def web_search(self, search_terms: List[str]) -> List[Dict]:
        """Search the web for terms."""
        return self._http_client._request("POST", "/v1/utilities/websearch", json={
            "search_terms": search_terms
        })


    def web_parser(self, urls: List[str]) -> List[Dict]:
        """Parse web pages for content."""
        return self._http_client._request("POST", "/v1/utilities/webparser", json={
            "urls": urls
        })



from .__http_client import _HTTPClient
from typing import Dict, List, Optional


class _KnowledgeBase:
    """Knowledge Base resource for managing vector databases."""

    def __init__(self, http_client: _HTTPClient):
        self._http_client = http_client


    def create(self, name: str, meta: Optional[Dict] = None) -> Dict:
        """Create a new knowledge base."""
        if not name:
            raise ValueError("Name cannot be empty")
        payload = {"name": name}
        if meta:
            payload["meta"] = meta
        return self._http_client._request("POST", "/v1/knowledgebase", json=payload)


    def list_knowledge_bases(self) -> Dict:
        """List all knowledge bases."""
        return self._http_client._request("GET", "/v1/knowledgebase")


    def get(self, knowledge_id: str) -> Dict:
        """Retrieve a knowledge base by ID."""
        if not knowledge_id:
            raise ValueError("Knowledge ID cannot be empty")
        return self._http_client._request("GET", f"/v1/knowledgebase/{knowledge_id}")


    def add_source(
        self,
        knowledge_id: str,
        type: str,
        meta: Optional[Dict] = None,
        urls: Optional[Dict] = None
    ) -> Dict:
        """Add sources to a knowledge base."""
        
        valid_types = ["docs", "web", "youtube"]
        if type not in valid_types:
            raise ValueError(f"Invalid type. Valid types: {self._http_client._stringify(valid_types)} .")

        payload = {
            "type": type,
            "options": {
                "urls": urls
            }
        }
        optional_params = {"meta": meta}
        for param in optional_params:
            param_value = optional_params[param]
            if param_value is not None:
                payload[param] = param_value
        return self._http_client._request("PUT", f"/v1/knowledgebase/{knowledge_id}", json=payload)


    def get_status(self, knowledge_id: str) -> Dict:
        """Get knowledge base processing status."""
        return self._http_client._request("GET", f"/v1/knowledgebase/{knowledge_id}/status")


    def get_source(self, knowledge_id: str, source_id: str) -> Dict:
        """Retrieve source details."""
        return self._http_client._request("GET", f"/v1/knowledgebase/{knowledge_id}/{source_id}")


    def list_sources(self, knowledge_id: str) -> List[Dict]:
        """List all sources in a knowledge base."""
        return self._http_client._request("GET", f"/v1/knowledgebase/{knowledge_id}/sources")


    def delete_sources(
        self,
        knowledge_id: str,
        values: List[str],
        delete_all: Optional[bool] = False
    ) -> Dict:
        """Delete sources from a knowledge base."""
        payload = {"values": values}
        if delete_all:
            payload["delete_all"] = delete_all
        return self._http_client._request("DELETE", f"/v1/knowledgebase/{knowledge_id}/source", json=payload)
